package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.query.EquipmentQuery;
import com.taauav.admin.vo.TauvEquipmentListVo;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.TauvEquipment;
import com.taauav.admin.mapper.TauvEquipmentMapper;
import com.taauav.admin.service.ITauvEquipmentService;
import com.taauav.common.service.impl.BaseServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 设备管理表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-13
 */
@Service
public class TauvEquipmentServiceImpl extends BaseServiceImpl<TauvEquipmentMapper, TauvEquipment> implements ITauvEquipmentService {
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Autowired
    private TauvEquipmentMapper equipmentMapper;
    @Autowired
    private Response response;

    /**
     * 获取飞行设备列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getPageList(EquipmentQuery query) {
        // 查询条件
        QueryWrapper<TauvEquipment> queryWrapper = new QueryWrapper<TauvEquipment>();
        // 设备编码
        if (!StringUtils.isEmpty(query.getEquipmentSn())) {
            queryWrapper.like("equipment_sn", query.getEquipmentSn());
        }
        // 设备名称
        if (!StringUtils.isEmpty(query.getName())) {
            queryWrapper.like("name", query.getName());
        }
        // 无人机类型
        if (query.getType() != null && query.getType() > 0) {
            queryWrapper.eq("type", query.getType());
        }
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByDesc("id");

        // 查询数据
        IPage<TauvEquipment> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvEquipment> data = equipmentMapper.selectPage(page, queryWrapper);
        List<TauvEquipment> equipmentList = data.getRecords();
        List<TauvEquipmentListVo> equipmentListVoList = new ArrayList<>();
        TauvEquipment tauvEquipment = new TauvEquipment();
        if (!equipmentList.isEmpty()) {
            equipmentList.forEach(item -> {
                TauvEquipmentListVo equipmentListVo = new TauvEquipmentListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, equipmentListVo);
                // 设置状态描述
                equipmentListVo.setStatusName(tauvEquipment.getStatusList().get(equipmentListVo.getStatus()));
                // 设置无人机类型名称
                equipmentListVo.setTypeName(tauvEquipment.getTypeList().get(equipmentListVo.getType()));
                equipmentListVoList.add(equipmentListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", equipmentListVoList);
        return response.success(result);
    }

    /**
     * 编辑设备数据
     *
     * @param equipment
     * @return
     */
    @Override
    public Response edit(TauvEquipment equipment) {
        if (equipment == null || equipment.getId() == null) {
            return response.failure("实体对象不能为空");
        }
        // 产品需要唯一性验证
        QueryWrapper<TauvEquipment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("product_sn", equipment.getProductSn());
        queryWrapper.eq("mark", 1);
        Integer count = equipmentMapper.selectCount(queryWrapper);
        if (count > 0) {
            return response.failure("产品序号已存在");
        }
        boolean result = this.editData(equipment);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 添加设备数据
     *
     * @param equipment
     * @return
     */
    @Override
    public Response add(TauvEquipment equipment) {
        if (equipment == null) {
            return response.failure("实体对象不能为空");
        }
        // 产品需要唯一性验证
        QueryWrapper<TauvEquipment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("product_sn", equipment.getProductSn());
        queryWrapper.eq("mark", 1);
        Integer count = equipmentMapper.selectCount(queryWrapper);
        if (count > 0) {
            return response.failure("产品序号已存在");
        }
        boolean result = this.addData(equipment);
        if (!result) {
            return response.failure("添加失败");
        }
        // 设置设备编码
        String equipmentSn = String.format("%05d", equipment.getId());
        equipment.setEquipmentSn(equipmentSn);
        int rs = equipmentMapper.updateById(equipment);
        if (rs == 0) {
            return response.failure("设备编码更新失败");
        }
        return response.success("操作成功");
    }

    /**
     * 删除设备
     *
     * @param id 设备ID
     * @return
     */
    @Override
    public Response drop(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("设备ID不存在");
        }
        TauvEquipment entity = equipmentMapper.selectById(id);
        if (entity == null) {
            return response.failure("设备信息不存在");
        }
        boolean result = this.deleteById(id, entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 修改状态
     *
     * @param map
     * @return
     */
    @Override
    public Response setRules(Map<String, String> map) {
        if (StringUtils.isEmpty(map)) {
            return response.failure("参数不能为空");
        }
        String idStr = map.getOrDefault("id", "");
        String statusStr = map.getOrDefault("status", "");
        if (StringUtils.isEmpty(idStr)) {
            return response.failure("编号不存在");
        }
        if (StringUtils.isEmpty(statusStr)) {
            return response.failure("状态不能为空");
        }
        if (!("1".equals(statusStr) || "2".equals(statusStr))) {
            return response.failure("状态值不合法");
        }
        TauvEquipment tauvEquipment = new TauvEquipment();
        tauvEquipment.setStatus(Integer.valueOf(statusStr));
        tauvEquipment.setId(Integer.valueOf(idStr));
        equipmentMapper.updateById(tauvEquipment);
        return response.success("操作成功");
    }

    /**
     * 根据编号获取数据
     *
     * @param id
     * @return
     */
    @Override
    public TauvEquipment getInfoById(Integer id) {
        if (StringUtils.isEmpty(id)) {
            return null;
        }
        return equipmentMapper.selectById(id);
    }

    /**
     * 分类和状态等数据
     *
     * @return
     */
    @Override
    public Map<String, Object> getBaseData() {
        TauvEquipment equipment = new TauvEquipment();
        Map<String, Object> map = new HashMap<>(2);
        map.put("type_list", equipment.getTypeList());
        map.put("task_type_list", equipment.getTaskTypeList());
        map.put("status_list", equipment.getStatusList());
        return map;
    }

    /**
     * 获取无人机列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getEquipmentList(EquipmentQuery query) {
        // 查询条件
        QueryWrapper<TauvEquipment> queryWrapper = new QueryWrapper<>();
        // 无人机类型
        if (query.getType() != null && query.getType() > 0) {
            queryWrapper.eq("type", query.getType());
        }
        // 无人机型号
        if (!StringUtils.isEmpty(query.getModel())) {
            queryWrapper.like("model", query.getModel());
        }
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByDesc("id");

        // 查询列表
        IPage<TauvEquipment> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvEquipment> data = equipmentMapper.selectPage(page, queryWrapper);
        List<TauvEquipment> equipmentList = data.getRecords();
        List<TauvEquipmentListVo> equipmentListVoList = new ArrayList<>();
        if (!equipmentList.isEmpty()) {
            equipmentList.forEach(item -> {
                TauvEquipmentListVo equipmentListVo = new TauvEquipmentListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, equipmentListVo);
                // 设备类型
                equipmentListVo.setTypeName(item.getTypeList().get(item.getType()));
                // 任务类型
                equipmentListVo.setTaskTypeName(this.getTaskTypeName(item.getTaskType()));
                // 状态名称
                equipmentListVo.setStatusName(item.getStatusList().get(item.getStatus()));
                equipmentListVoList.add(equipmentListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", equipmentListVoList);
        return response.success(result);
    }

    /**
     * 获取无人机设备名称
     *
     * @param equipmentIds 无人机设备ID
     * @return
     */
    @Override
    public List<String> getEquipmentNameByIds(String equipmentIds) {
        // 查询条件
        QueryWrapper<TauvEquipment> queryWrapper = new QueryWrapper<>();
        // 待分配任务
        String arr[] = equipmentIds.split(",");
        ArrayList arrayList = new ArrayList();
        for (String s : arr) {
            arrayList.add(s);
        }
        queryWrapper.in("id", arrayList);
        queryWrapper.eq("mark", 1);

        // 查询数据
        List<TauvEquipment> equipmentList = equipmentMapper.selectList(queryWrapper);
        List<String> stringList = new ArrayList<>();
        if (!equipmentList.isEmpty()) {
            equipmentList.forEach(item -> {
                stringList.add(item.getName());
            });
        }
        return stringList;
    }

    /**
     * 根据设备ID获取设备列表
     *
     * @param equipmentIds 设备ID
     * @return
     */
    @Override
    public List<TauvEquipmentListVo> getEquipmentListByIds(String equipmentIds) {
        // 查询条件
        QueryWrapper<TauvEquipment> queryWrapper = new QueryWrapper<>();
        // 待分配任务
        String arr[] = equipmentIds.split(",");
        ArrayList arrayList = new ArrayList();
        for (String s : arr) {
            arrayList.add(s);
        }
        queryWrapper.in("id", arrayList);
        queryWrapper.eq("mark", 1);

        // 查询数据
        List<TauvEquipment> equipmentList = equipmentMapper.selectList(queryWrapper);
        List<TauvEquipmentListVo> equipmentListVoList = new ArrayList<>();
        if (!equipmentList.isEmpty()) {
            equipmentList.forEach(item -> {
                TauvEquipmentListVo equipmentListVo = new TauvEquipmentListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, equipmentListVo);
                // 设备类型
                equipmentListVo.setTypeName(item.getTypeList().get(item.getType()));
                // 任务类型
                equipmentListVo.setTaskTypeName(this.getTaskTypeName(item.getTaskType()));
                // 状态名称
                equipmentListVo.setStatusName(item.getStatusList().get(item.getStatus()));
                equipmentListVoList.add(equipmentListVo);
            });
        }
        return equipmentListVoList;
    }

    /**
     * 根据任务类型ID获取任务名称
     *
     * @param taskType 任务类型
     * @return
     */
    public String getTaskTypeName(String taskType) {
        List<String> stringList = new ArrayList<>();
        String[] itemArr = taskType.split(",");
        TauvEquipment equipment = new TauvEquipment();
        for (String s : itemArr) {
            String typeStr = equipment.getTaskTypeList().get(Integer.valueOf(s));
            stringList.add(typeStr);
        }
        return org.apache.commons.lang3.StringUtils.join(stringList.toArray(), ",");
    }
}
