package com.taauav.common.util;

import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class ExcelUtil {

	public static String exportExcelFile(List<List<String>> excelData, String path, String sheetName, int columnWidth) {
		HSSFWorkbook workbook = new HSSFWorkbook();
		HSSFSheet sheet = workbook.createSheet(sheetName);

		if (columnWidth > 0) {
			sheet.setDefaultColumnWidth(columnWidth);
		}
		sheet.setDefaultRowHeightInPoints(18);

		int rowIndex = 0;

		for (List<String> data : excelData) {
			//创建一个row行，然后增值1
			HSSFRow row = sheet.createRow(rowIndex++);
			//遍历添加本行数据
			for (int i = 0; i < data.size(); i++) {
				HSSFCell cell = row.createCell(i);
				//创建一个内容对象
				HSSFRichTextString text = new HSSFRichTextString(data.get(i));
				//将内容对象的文字内容写入到单元格
				cell.setCellValue(text);
			}
		}
		FileOutputStream outputStream;
		try {
			outputStream = new FileOutputStream(path);
			workbook.write(outputStream);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return path;
	}

	/**
	 * 导入数据（单页）
	 *
	 * @param file        文件
	 * @param sheetIndex  页名的索引(从0开始，-1代表全部页)
	 * @param headerIndex 表头的索引（用于获取共多少列以及第几行开始读数据）
	 * @return
	 * @throws IOException
	 */
	public static List<List<Object>> importExcel(MultipartFile file, int sheetIndex, int headerIndex) throws Exception {
		Workbook workbook = null;
		//返回的data
		List<List<Object>> data = new ArrayList<>();
		workbook = getWorkbook(file);
		//导入某一页
		if (sheetIndex != -1 && sheetIndex > -1) {
			Sheet sheet = workbook.getSheetAt(sheetIndex);
			List<List<Object>> lists = importOneSheet(sheet, headerIndex);
			data.addAll(lists);
		} else {
			//导入全部
			for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
				Sheet sheet = workbook.getSheetAt(i);
				if (sheet == null) {
					continue;
				}
				List<List<Object>> lists = importOneSheet(sheet, headerIndex);
				data.addAll(lists);
			}
		}
		return data;
	}

	/**
	 * 导入数据（所有页）
	 *
	 * @param file        文件
	 * @param headerIndex 表头的索引（用于获取共多少列以及第几行开始读数据）
	 * @return
	 * @throws IOException
	 */
	public static List<List<Object>> importExcel(MultipartFile file, int headerIndex) throws Exception {
		return importExcel(file, -1, headerIndex);
	}

	/**
	 * 获取表头
	 *
	 * @param file
	 * @param headerIndex
	 * @return
	 */
	public static List<String> getHeadList(MultipartFile file, Integer sheetIndex, Integer headerIndex) throws Exception {
		Workbook workbook = null;
		workbook = getWorkbook(file);
		Sheet sheet = workbook.getSheetAt(sheetIndex);
		Integer columnNum = sheet.getRow(headerIndex).getPhysicalNumberOfCells();
		Row row = sheet.getRow(headerIndex);
		List<String> headList = new ArrayList<>();
		for (Integer m = 0; m < columnNum; m++) {
			headList.add(getCellValue(row.getCell(m)));
		}
		return headList;
	}

	/**
	 * 获取一个sheet里的数据
	 *
	 * @param sheet
	 * @param headerIndex
	 * @return
	 * @throws Exception
	 */
	private static List<List<Object>> importOneSheet(Sheet sheet, int headerIndex) throws Exception {
		List<List<Object>> data = new ArrayList<>();
		int row = sheet.getLastRowNum();
		//row = -1 表格中没有数据
		//row = headerIndex 表格中表头以下没有数据（指没有有用数据）
		if (row == -1 || row == headerIndex) {
			throw new Exception("表格中没有有用数据!");
		}
		//通过表头获取共多少列
		int coloumNum = sheet.getRow(headerIndex).getPhysicalNumberOfCells();
		//从表头下一行开始取数据
		for (int i = headerIndex + 1; i <= row; i++) {
			Row row1 = sheet.getRow(i);
			List<Object> list = new ArrayList<>();
			if (row1 != null) {
				for (int j = 0; j < coloumNum; j++) {
					list.add(getCellValue(row1.getCell(j)));
				}
			}
			data.add(list);
		}
		return data;
	}

	/**
	 * 获取workbook
	 *
	 * @return
	 */
	private static Workbook getWorkbook(MultipartFile file) throws Exception {
		Workbook workbook = null;
		//获取文件名
		String fileName = file.getOriginalFilename();
		//判断文件格式
		if (fileName.endsWith("XLS") || fileName.endsWith("xls")) {
			workbook = new HSSFWorkbook(file.getInputStream());
		} else if (fileName.endsWith("XLSX") || fileName.endsWith("xlsx")) {
			workbook = new XSSFWorkbook(file.getInputStream());
		} else {
			throw new Exception("文件格式有误!");
		}
		return workbook;
	}


	/**
	 * 获取单元格的值
	 *
	 * @param cell
	 * @return
	 */
	private static String getCellValue(Cell cell) {
		if (StringUtils.isEmpty(cell)) {
			return "";
		}
		CellType cellType = cell.getCellTypeEnum();
		String cellValue = "";
		switch (cellType) {
			// 数字
			case NUMERIC:
				DecimalFormat df = new DecimalFormat("0.00");
				cellValue = df.format(cell.getNumericCellValue());
				break;
			// 字符串
			case STRING:
				cellValue = cell.getStringCellValue();
				break;
			// Boolean
			case BOOLEAN:
				cellValue = cell.getBooleanCellValue() + "";
				break;
			// 公式
			case FORMULA:
				cellValue = cell.getCellFormula() + "";
				break;
			// 空值
			case BLANK:
				cellValue = "";
				break;
			// 故障
			case ERROR:
				cellValue = "非法字符";
				break;
			default:
				cellValue = "未知类型";
				break;
		}
		return cellValue.trim();
	}

	/**
	 * 根据模板导出数据
	 * @param templatePath 模板路径（包括文件名）
	 * @param filePath 导出文件路径（包括文件名）
	 * @param map 直接渲染数据
	 * @param list 遍历渲染数组
	 * @return
	 * @throws IOException
	 */
	public static String exportExcelByTemplate(String templatePath, String filePath, Map<String, String> map, List<Map<String, String>> list) throws IOException {
		//复制模板到新文件
		File template = new File(templatePath);
		File file = new File(filePath);
		if (!file.exists()) {
			Files.copy(template.toPath(), file.toPath());
		} else {
			file.delete();
			Files.copy(template.toPath(), file.toPath());
		}

		FileOutputStream outputStream;
		try {
			POIFSFileSystem fileSystem = new POIFSFileSystem(file);
			HSSFWorkbook workbook = new HSSFWorkbook(fileSystem);
			Integer sheetNum = workbook.getNumberOfSheets();
			if (sheetNum > 0) {
				for (int num = 0; num < sheetNum; num++) {
					HSSFSheet sheet = workbook.getSheetAt(num);
					//循环map中的键值对，替换excel中对应的键的值（注意，excel模板中的要替换的值必须跟map中的key值对应，不然替换不成功）
					if (!StringUtils.isEmpty(map)) {
						for (Object atr : map.keySet()) {
							//该sheet页里最多有几行内容
							int rowNum = sheet.getLastRowNum();
							String atrStr = "${" + atr + "}";
							for (int i = 0; i < rowNum; i++) {
								HSSFRow row = sheet.getRow(i);
								int colNum = row.getLastCellNum();
								for (int j = 0; j < colNum; j++) {
									HSSFCell cell = row.getCell((short) j);
									String str = cell.getStringCellValue();
									if (str.startsWith("$") && atrStr.equals(str)) {
										//写入单元格内容
										cell.setCellValue(map.get(atr));
									}
								}
							}
						}
					}
					if (!StringUtils.isEmpty(list)) {
						Map<String, String> item = list.get(0);
						int m = -1;
						here:
						for (Object atr : item.keySet()) {
							int rowNum = sheet.getLastRowNum();
							atr = "${item." + atr + "}";
							for (int i = 0; i < rowNum; i++) {
								HSSFRow row = sheet.getRow(i);
								int colNum = row.getLastCellNum();
								for (int j = 0; j < colNum; j++) {
									HSSFCell cell = row.getCell((short) j);
									String str = cell.getStringCellValue();
									if (str.startsWith("$") && atr.equals(str)) {
										m = i;
										break here;
									}
								}
							}
						}
						if (m >= 0) {
							for (int n = 0; n < list.size(); n++) {
								if (n < list.size() - 1) {
									insertRow(sheet, m, 1);
								}
								HSSFRow row = sheet.getRow(m);
								int colNum = row.getLastCellNum();
								Map<String, String> data = list.get(n);
								for (Object word : data.keySet()) {
									String wordStr = "${item." + word + "}";
									for (int j = 0; j < colNum; j++) {
										HSSFCell cell = row.getCell((short) j);
										String str = cell.getStringCellValue();
										if (str.startsWith("$") && wordStr.equals(str)) {
											//写入单元格内容
											cell.setCellValue(data.get(word));
											break;
										}
									}
								}
								m++;
							}
						}
					}
				}
			}
			// 输出文件
			outputStream = new FileOutputStream(filePath);
			workbook.write(outputStream);
			outputStream.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return filePath;
	}

	/**
	 * 插入行
	 * insert row into the target sheet, the style of cell is the same as startRow
	 *
	 * @param sheet
	 * @param starRow
	 * @param rows
	 */
	public static void insertRow(HSSFSheet sheet, int starRow, int rows) {
		sheet.shiftRows(starRow + 1, sheet.getLastRowNum(), rows, true, false);
		starRow = starRow - 1;

		for (int i = 0; i < rows; i++) {
			HSSFRow sourceRow = null;
			HSSFRow targetRow = null;
			HSSFCell sourceCell = null;
			HSSFCell targetCell = null;
			short m;

			starRow = starRow + 1;
			sourceRow = sheet.getRow(starRow);
			targetRow = sheet.createRow(starRow + 1);
			targetRow.setHeight(sourceRow.getHeight());

			for (m = sourceRow.getFirstCellNum(); m < sourceRow.getLastCellNum(); m++) {
				sourceCell = sourceRow.getCell(m);
				targetCell = targetRow.createCell(m);
				targetCell.setCellStyle(sourceCell.getCellStyle());
				targetCell.setCellValue(sourceCell.getStringCellValue());
			}
		}
	}
}
