package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.SysAuthGroup;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.StringUtils;
import com.taauav.front.entity.UserAuthGroup;
import com.taauav.front.mapper.UserAuthGroupMapper;
import com.taauav.front.query.UserAuthGroupQuery;
import com.taauav.front.service.IUserAuthGroupService;
import com.taauav.front.vo.UserAuthGroupListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 外部用户角色表 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-14
 */
@Service
public class UserAuthGroupServiceImpl extends BaseServiceImpl<UserAuthGroupMapper, UserAuthGroup> implements IUserAuthGroupService {

    @Autowired
    private UserAuthGroupMapper userAuthGroupMapper;

    @Autowired
    private Response response;

    /**
     * 获取数据列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(UserAuthGroupQuery query) {
        // 查询条件
        QueryWrapper<UserAuthGroup> queryWrapper = new QueryWrapper<>();
        // 角色名称
        if (!StringUtils.isEmpty(query.getName())) {
            queryWrapper.like("name", query.getName());
        }
        queryWrapper.eq("mark", 1);

        IPage<UserAuthGroup> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<UserAuthGroup> data = userAuthGroupMapper.selectPage(page, queryWrapper);
        List<UserAuthGroup> authGroupList = data.getRecords();
        List<UserAuthGroupListVo> authGroupListVoList = new ArrayList<>();
        if (!authGroupList.isEmpty()) {
            authGroupList.forEach(item -> {
                UserAuthGroupListVo userAuthGroupListVo = new UserAuthGroupListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, userAuthGroupListVo);
                // 状态
                userAuthGroupListVo.setStatusName(item.getStatus() == 1 ? "正常" : "禁用");
                authGroupListVoList.add(userAuthGroupListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", authGroupListVoList);
        return response.success(result);
    }

    /**
     * 添加角色
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(UserAuthGroup entity) {
        if (entity == null) {
            return response.failure("角色信息不存在");
        }
        boolean result = this.addData(entity);
        if (!result) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取详情
     *
     * @param id 角色ID
     * @return
     */
    @Override
    public Response info(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("角色ID不存在");
        }
        UserAuthGroup entity = userAuthGroupMapper.selectById(id);
        return response.success(entity);
    }

    /**
     * 编辑角色
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(UserAuthGroup entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("角色ID不存在");
        }
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除角色
     *
     * @param id 角色ID
     * @return
     */
    @Override
    public Response drop(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("角色ID不存在");
        }
        UserAuthGroup entity = userAuthGroupMapper.selectById(id);
        if (entity == null) {
            return response.failure("角色信息不存在");
        }
        boolean result = this.deleteById(id, entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(UserAuthGroup entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("角色ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("角色状态不能为空");
        }
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

    /**
     * 获取角色列表
     *
     * @return
     */
    @Override
    public Response getAuthGroupList() {
        // 查询条件
        QueryWrapper<UserAuthGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询数据
        List<UserAuthGroup> UserAuthGroupList = userAuthGroupMapper.selectList(queryWrapper);
        return response.success(UserAuthGroupList);
    }

    /**
     * 获取外部角色
     *
     * @param authGroupIds 角色ID
     * @return
     */
    @Override
    public String getAuthGroupName(String authGroupIds) {
        // 查询条件
        QueryWrapper<UserAuthGroup> queryWrapper = new QueryWrapper<>();
        // 待分配任务
        String arr[] = authGroupIds.split(",");
        ArrayList arrayList = new ArrayList();
        for (String s : arr) {
            arrayList.add(s);
        }
        queryWrapper.in("id", arrayList);
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询数据
        List<UserAuthGroup> UserAuthGroupList = userAuthGroupMapper.selectList(queryWrapper);
        List<String> stringList = new ArrayList<>();
        if (!UserAuthGroupList.isEmpty()) {
            UserAuthGroupList.forEach(item -> {
                stringList.add(item.getName());
            });
        }
        return org.apache.commons.lang3.StringUtils.join(stringList.toArray(), " ");
    }

    /**
     * 根据用户编号获取权限
     *
     * @param adminId
     * @return
     */
    @Override
    public List<UserAuthGroup> getAuthGroups(Integer adminId) {
        return userAuthGroupMapper.getAuthGroups(adminId);
    }

}
