package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import com.taauav.front.constant.UserNoticeConstant;
import com.taauav.front.entity.UserAdmin;
import com.taauav.front.entity.UserNotice;
import com.taauav.front.mapper.UserAdminMapper;
import com.taauav.front.mapper.UserNoticeMapper;
import com.taauav.front.query.UserNoticeQuery;
import com.taauav.front.service.IUserNoticeService;
import com.taauav.front.vo.usernotice.UserNoticeInfoVo;
import com.taauav.front.vo.usernotice.UserNoticeListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 通知公告表 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-14
 */
@Service
public class UserNoticeServiceImpl extends BaseServiceImpl<UserNoticeMapper, UserNotice> implements IUserNoticeService {

    @Autowired
    private UserNoticeMapper noticeMapper;

    @Autowired
    private Response response;

    @Autowired
    private UserAdminMapper userAdminMapper;

    /**
     * 获取通知公告列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getNoticeList(UserNoticeQuery query) {
        // 查询条件
        QueryWrapper<UserNotice> queryWrapper = new QueryWrapper<>();
        // 标题
        if (!StringUtils.isEmpty(query.getTitle())) {
            queryWrapper.like("title", query.getTitle());
        }
        // 公告类型
        if (query.getType() != null && query.getType() > 0) {
            queryWrapper.eq("type", query.getType());
        }
        // 发布人
        if (query.getPublishUser() != null && query.getPublishUser() > 0) {
            queryWrapper.eq("publish_user", query.getPublishUser());
        }
        // 发布开始时间
        if (!StringUtils.isEmpty(query.getStartTime())) {
            Date startTime = DateUtil.parse(query.getStartTime(), "yyyy-MM-dd HH:mm:ss");
            queryWrapper.ge("publish_time", startTime);
        }
        // 发布结束时间
        if (!StringUtils.isEmpty(query.getEndTime())) {
            Date endTime = DateUtil.parse(query.getEndTime(), "yyyy-MM-dd HH:mm:ss");
            queryWrapper.le("publish_time", endTime);
        }

        // 查询数据
        IPage<UserNotice> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<UserNotice> data = noticeMapper.selectPage(page, queryWrapper);
        List<UserNotice> noticeList = data.getRecords();
        List<UserNoticeListVo> userNoticeListVoList = new ArrayList<>();
        if (!noticeList.isEmpty()) {
            noticeList.forEach(item -> {
                UserNoticeListVo userNoticeListVo = new UserNoticeListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, userNoticeListVo);
                // 发布人名称
                UserAdmin userAdmin = userAdminMapper.selectById(item.getPublishUser());
                if (userAdmin != null) {
                    userNoticeListVo.setPublishUserName(userAdmin.getRealname());
                }
                // 通知状态
                userNoticeListVo.setStatusName(UserNoticeConstant.USER_NOTICE_STATUS_LIST.get(item.getStatus()));
                // 通知类型
                userNoticeListVo.setTypeName(UserNoticeConstant.USER_NOTICE_TYPE_LIST.get(item.getType()));
                userNoticeListVoList.add(userNoticeListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", userNoticeListVoList);
        return response.success(result);
    }

    /**
     * 添加通知公告
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(UserNotice entity) {
        if (entity == null) {
            return response.failure("实体对象不能为空");
        }
        // 设置发布人和发布时间
        if (entity.getStatus() == 1) {
            entity.setPublishUser(ShiroUtils.getAdminId());
            entity.setPublishTime(DateUtil.now());
        }
        boolean result = this.addData(entity);
        if (!result) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 编辑通知公告
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(UserNotice entity) {
        if (entity == null) {
            return response.failure("实体对象不能为空");
        }
        if (entity.getId() == null && entity.getId() < 0) {
            return response.failure("记录ID不能为空");
        }
        // 设置发布人和发布时间
        if (entity.getStatus() == 1) {
            entity.setPublishUser(ShiroUtils.getAdminId());
            entity.setPublishTime(DateUtil.now());
        }
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 获取通知公告详情
     *
     * @param noticeId 通知公告ID
     * @return
     */
    @Override
    public Response info(Integer noticeId) {
        if (noticeId == null || noticeId <= 0) {
            return response.failure("公告ID不能为空");
        }
        UserNotice info = this.getInfo(noticeId);
        // 拷贝属性
        UserNoticeInfoVo userNoticeInfoVo = new UserNoticeInfoVo();
        if (info != null) {
            BeanUtils.copyProperties(info, userNoticeInfoVo);
            // 发布人
            UserAdmin userAdmin = userAdminMapper.selectById(info.getPublishUser());
            if (userAdmin != null) {
                userNoticeInfoVo.setPublishUserName(userAdmin.getRealname());
            }
        }
        return response.success(userNoticeInfoVo);
    }

    /**
     * 删除通知公告
     *
     * @param noticeId 通知公告ID
     * @return
     */
    @Override
    public Response delete(Integer noticeId) {
        if (noticeId == null || noticeId <= 0) {
            return response.failure("通知公告ID不能为空");
        }
        UserNotice info = this.getInfo(noticeId);
        if (info == null) {
            return response.failure("通知公告信息不存在");
        }
        boolean result = this.deleteById(noticeId, info);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }
}
